/** @format */

// Import test packages
import axios from "axios";
import http from "axios/lib/adapters/http";
import jestOpenAPI from "jest-openapi";
import { StatusCodes } from "http-status-codes";

// Load an OpenAPI file (YAML or JSON) into this plugin
jestOpenAPI(process.env.PWD + "/openapi/API.yaml");

// Set API version to use
const api = "erp/api/v0";

// Access to the database test user
const server = process.env.LSMB_BASE_URL;
const username = process.env.UIUSER;
const password = process.env.UIPASSWORD;
const company = process.env.LSMB_NEW_DB_API;

let headers = {};

// For all tests
beforeAll(async () => {
    axios.defaults.adapter = http;
});

// Log in before each test
beforeEach(async () => {
    let r = await axios.post(
        server + "/login.pl?action=authenticate&company=" + encodeURI(company),
        {
            company: company,
            password: password,
            login: username
        },
        {
            headers: {
                "X-Requested-With": "XMLHttpRequest",
                "Content-Type": "application/json"
            }
        }
    );
    if (r.status === StatusCodes.OK) {
        headers = {
            cookie: r.headers["set-cookie"],
            referer: server + "/" + r.data.target,
            authorization: "Basic " + btoa(username + ":" + password)
        };
    }
});
// Log out after each test
afterEach(async () => {
    let r = await axios.get(server + "/login.pl?action=logout&target=_top");
    if (r.status === StatusCodes.OK) {
        headers = {};
    }
});

// Language tests
describe("Retrieving all languages", () => {
    it("GET /languages should satisfy OpenAPI spec", async () => {
        // Get an HTTP response from your server
        let res = await axios.get(server + "/" + api + "/languages", {
            headers: headers
        });
        expect(res.status).toEqual(StatusCodes.OK);

        // Assert that the HTTP response satisfies the OpenAPI spec
        expect(res).toSatisfyApiSpec();
    });
});

describe("Retrieving all languages with old syntax should fail", () => {
    it("GET /languages/ should fail", async () => {
        await expect(
            axios.get(server + "/" + api + "/languages/", {
                headers: headers
            })
        ).rejects.toThrow(
            "Request failed with status code " + StatusCodes.BAD_REQUEST
        );
    });
});

describe("Retrieve English language", () => {
    it("GET /language/en should work and satisfy the OpenAPI spec", async () => {
        let res = await axios.get(server + "/" + api + "/languages/en", {
            headers: headers
        });
        expect(res.status).toEqual(StatusCodes.OK);

        // Assert that the HTTP response satisfies the OpenAPI spec
        expect(res).toSatisfyApiSpec();

        // Assert that the HTTP response satisfies the OpenAPI spec
        expect(res.data).toSatisfySchemaInApiSpec("Language");
    });
});

describe("Retrieve non-existant Navaho language", () => {
    it("GET /languages/nv should not retrieve Navajo language", async () => {
        await expect(
            axios.get(server + "/" + api + "/languages/nv", {
                headers: headers
            })
        ).rejects.toThrow(
            "Request failed with status code " + StatusCodes.NOT_FOUND
        );
    });
});

describe("Adding the new Navaho Language", () => {
    it("POST /languages/nv should allow adding Navaho language", async () => {
        let res = await axios.post(
            server + "/" + api + "/languages",
            {
                code: "nv",
                description: "Navaho"
            },
            {
                headers: headers
            }
        );
        expect(res.status).toEqual(StatusCodes.CREATED);

        // Assert that the HTTP response satisfies the OpenAPI spec
        expect(res.data).toSatisfySchemaInApiSpec("Language");
    });
});

describe("Modifying the new Navajo language", () => {
    it("PUT /languages/nv should allow updating Navajo language", async () => {
        let res = await axios.get(server + "/" + api + "/languages/nv", {
            headers: headers
        });
        expect(res.status).toEqual(StatusCodes.OK);
        expect(res.headers.etag).toBeDefined();
        res = await axios.put(
            server + "/" + api + "/languages/nv",
            {
                code: "nv",
                description: "Navajo"
            },
            {
                headers: { ...headers, "If-Match": res.headers.etag }
            }
        );
        expect(res.status).toEqual(StatusCodes.OK);

        // Assert that the HTTP response satisfies the OpenAPI spec
        expect(res).toSatisfyApiSpec();

        // Assert that the HTTP response satisfies the OpenAPI spec
        expect(res.data).toSatisfySchemaInApiSpec("Language");
    });
});

/*
 * Not implemented yet
describe("Updating the new Navaho language", () => {
    it("PATCH /languages/nv should allow updating Navajo language", async () => {
        let res = await axios.get(server + "/" + api + "/languages/nv", {
            headers: headers
        });
        expect(res.status).toEqual(StatusCodes.OK);
        expect(res.headers.etag).toBeDefined();
        res = await axios.patch(
            server + "/" + api + "/languages/nv",
            {
                code: "nv",
                description: "Navaho"
            },
            {
                headers: { ...headers, "If-Match": res.headers.etag }
            }
        );
        expect(res.status).toEqual(StatusCodes.OK);

        // Assert that the HTTP response satisfies the OpenAPI spec
        expect(res).toSatisfyApiSpec();

        // Assert that the HTTP response satisfies the OpenAPI spec
        expect(res.data).toSatisfySchemaInApiSpec("Language");
    });
});
*/

describe("Not removing the new Navajo language", () => {
    it("DELETE /languages/nv should allow deleting Navajo language", async () => {
        let res = await axios.get(server + "/" + api + "/languages/nv", {
            headers: headers
        });
        expect(res.status).toEqual(StatusCodes.OK);
        expect(res.headers.etag).toBeDefined();

        await expect(
            axios.delete(server + "/" + api + "/languages/nv", {
                headers: { ...headers, "If-Match": res.headers.etag }
            })
        ).rejects.toThrow(
            "Request failed with status code " + StatusCodes.FORBIDDEN
        );
    });
});
